local _G = getfenv(0)
local dewdrop = AceLibrary("Dewdrop-2.0")
local surface = AceLibrary("Surface-1.0")
local L = AceLibrary("AceLocale-2.2"):new("XPBarNone")

-- Register our textures.
surface:Register("BantoBar", "Interface\\AddOns\\XPBarNone\\Textures\\bantobar")
surface:Register("Blizzard", "Interface\\TargetingFrame\\UI-StatusBar")
surface:Register("Glaze", "Interface\\AddOns\\XPBarNone\\Textures\\glaze")
surface:Register("Marble", "Interface\\AddOns\\XPBarNone\\Textures\\marble")
surface:Register("Otravi", "Interface\\AddOns\\XPBarNone\\Textures\\otravi")
surface:Register("Perl", "Interface\\AddOns\\XPBarNone\\Textures\\perl")
surface:Register("Smooth", "Interface\\AddOns\\XPBarNone\\Textures\\smooth")
surface:Register("Steel", "Interface\\AddOns\\XPBarNone\\Textures\\steel")
surface:Register("Striped", "Interface\\AddOns\\XPBarNone\\Textures\\striped")
surface:Register("Waves", "Interface\\AddOns\\XPBarNone\\Textures\\waves")

XPBarNone = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
XPBarNone:RegisterDB("XPBarNoneDB")
XPBarNone:RegisterDefaults('profile', {
	rested = { r = 0, g = 0.4, b = 1 },
	resting = { r = 1.0, g = 0.82, b = 0.25 },
	normal = { r = 0.8, g = 0, b = 1 },
	remaining = { r = 0.82, g = 0, b = 0 },
	Border = false,
	Texture = "Smooth",
	Width = 1028,
	Height = 20,
	FontSize = 14,
	FontOutLine = false,
	PosX = nil,
	PosY = nil,
	Scale = 1,
	Strata = "HIGH",
	KillsToLevel = true,
	--PercentThroughLevel = true,
	--XPCurr = true,
	--XPLeft = true,
	--HideBlizzXPBar = true,
	HideText = false,
	IndicateRest = true,
	ShowRemaining = true,
	ShowReputation = false,
	MouseOver = false,
	Locked = false,
	RestedAsPercent = false,
	Bubbles = false
})

function XPBarNone:OnInitialize()
	self.options = {
		desc = L["Lightweight XP Bar."],
		type = "group",
		args = {
			general = {
				name = L["General"],
				desc = L["General options"],
				type = "group",
				args = {
					lock = {
						name = L["Lock"],
						desc = L["Toggle the locking."],
						type = "toggle",
						get = function() return self.db.profile.Locked end,
						set = function(val) self.db.profile.Locked = val end,
					},
					border = {
						name = L["Border"],
						desc = L["Toggle the border."],
						type = "toggle",
						get = function() return self.db.profile.Border end,
						set = function(val)
							self.db.profile.Border = val
							self:ToggleBorder()
						end,
					},
					bubbles = {
						name = L["Bubbles"],
						desc = L["Toggle bubbles on the XP bar."],
						type = "toggle",
						get = function() return self.db.profile.Bubbles end,
						set = function(val)
							self.db.profile.Bubbles = val
							self:ToggleBubbles()
						end,
					},
					scale = {
						name = L["Scale"],
						desc = L["Set the bar scale."],
						type = "range",
						min = 0.5,
						max = 2,
						get = function() return self.db.profile.Scale end,
						set = function(val)
							self:SavePosition()
							XPBarNoneFrame:SetScale(val)
							self.db.profile.Scale = val
							self:RestorePosition()
						end,
					},
					width = {
						name = L["Width"],
						desc = L["Set the bar width."],
						type = "range",
						min = 100,
						max = 2000,
						step = 1,
						get = function() return self.db.profile.Width end,
						set = function(val) 
							self.db.profile.Width = val
							XPBarNoneFrame:SetWidth(val)
							XPBarNoneB:SetWidth(val-4)
							XPBarNoneA:SetWidth(val-4)
							XPBarNoneR:SetWidth(val-4)
							XPBarNoneBubbles:SetWidth(val-4)
						end,

					},
					height = {
						name = L["Height"],
						desc = L["Set the bar height."],
						type = "range",
						min = 10,
						max = 100,
						step = 1,
						get = function() return self.db.profile.Height end,
						set = function(val)
							self.db.profile.Height = val
							XPBarNoneFrame:SetHeight(val)
							XPBarNoneB:SetHeight(val-8)
							XPBarNoneA:SetHeight(val-8)
							XPBarNoneR:SetHeight(val-8)
							XPBarNoneBubbles:SetHeight(val-8)
						end,
					},
					fontsize = {
						name = L["Font Size"],
						desc = L["Change the size of the text."],
						type = "range",
						min = 5,
						max = 30,
						step = 1,
						get = function() return self.db.profile.FontSize end,
						set = function(val)
							self.db.profile.FontSize = val
							self:SetFontOptions()
						end,
					},
					fontoutline = {
						name = L["Font Outline"],
						desc = L["Toggles the font outline."],
						type = "toggle",
						get = function() return self.db.profile.FontOutLine end,
						set = function(val)
							self.db.profile.FontOutLine = val
							self:SetFontOptions()
						end,
					},
					mouseover = {
						name = L["Mouse Over"],
						desc = L["Toggles switching between XP bar and Rep bar when you mouse over XPBarNone."],
						type = "toggle",
						get = function() return self.db.profile.MouseOver end,
						set = function(val) self.db.profile.MouseOver = val end,
					},
					strata = {
						name = L["Frame Strata"],
						desc = L["Set the frame strata."],
						type = "text",
						get = function() return self.db.profile.Strata end,
						set = function(val)
							self.db.profile.Strata = val
							XPBarNoneFrame:SetFrameStrata(val)
						end,
						validate = { "HIGH", "MEDIUM", "LOW", "BACKGROUND" },
					},
					texture = {
						name = L["Texture"],
						desc = L["Set the bar texture."],
						type = "text",
						get = function() return self.db.profile.Texture end,
						set = function(val)
							self.db.profile.Texture = val
							self:SetTexture(val)
						end,
						validate = surface:List(),
					},
					hidetext = {
						name = L["Hide Text"],
						desc = L["Hide the text on the XP and Rep bars."],
						type = "toggle",
						get = function() return self.db.profile.HideText end,
						set = function(val)
							self.db.profile.HideText = val
							self:UpdateXPBar()
						end,
					},
				},
			},
			xp = {
				name = L["Experience"],
				desc = L["Experience Bar related options"],
				type = "group",
				args = {
					text = {
						name = L["Text"],
						desc = L["Toggle various parts of the XP Bar text."],
						type = "group",
						args = {
							--cxp = {
							--	name = L["Current XP"],
							--	desc = L["Toggle showing the 'curxp/maxxp/(restxp)' text."],
							--	type = "toggle",
							--	get = function() return self.db.profile.XPCurr end,
							--	set = function(val) self.db.profile.XPCurr = val self:UpdateXPBar() end,
							--},
							ktl = {
								name = L["Kills to Level"],
								desc = L["Toggle showing kills to level."],
								type = "toggle",
								get = function() return self.db.profile.KillsToLevel end,
								set = function(val) self.db.profile.KillsToLevel = val self:UpdateXPBar() end,
							},
							--ptl = {
							--	name = L["Percent through level"],
							--	desc = L["Toggle showing the '% through level ?' text."],
							--	type = "toggle",
							--	get = function() return self.db.profile.PercentThroughLevel end,
							--	set = function(val) self.db.profile.PercentThroughLevel = val self:UpdateXPBar() end,
							--},
							--xpl = {
							--	name = L["XP left"],
							--	desc = L["Toggle showing the '??? XP Left' text."],
							--	type = "toggle",
							--	get = function() return self.db.profile.XPLeft end,
							--	set = function(val) self.db.profile.XPLeft = val self:UpdateXPBar() end,
							--},
							restedpercent = {
								name = L["Rested Percentage"],
								desc = L["Toggle between showing the raw rested XP value or showing it as a percentage."],
								type = "toggle",
								get = function() return self.db.profile.RestedAsPercent end,
								set = function(val) self.db.profile.RestedAsPercent = val self:UpdateXPBar() end,
							},
						},
					},
					remaining = {
						name = L["Remaining Rested XP"],
						desc = L["Toggle the display of remaining rested XP."],
						type = "toggle",
						get = function() return self.db.profile.ShowRemaining end,
						set = function(val) self.db.profile.ShowRemaining = val self:UpdateXPBar() end,
					},
					resting = {
						name = L["Rest Indication"],
						desc = L["Toggle the rest indication."],
						type = "toggle",
						get = function() return self.db.profile.IndicateRest end,
						set = function(val) self.db.profile.IndicateRest = val self:UpdateXPBar() end,
					},
					col = {
						name = L["Colours"],
						desc = L["Set the various bar colours."],
						type = "group",
						args = {
							normal = {
								name = L["Normal"],
								desc = L["Set the colour of the normal bar."],
								type = "color",
								get = function()
									return self.db.profile.normal.r, self.db.profile.normal.g, self.db.profile.normal.b
								end,
								set = function(r, g, b)
									self.db.profile.normal.r, self.db.profile.normal.g, self.db.profile.normal.b = r, g, b
									self:UpdateXPBar()
								end,
								hasAlpha = false
							},
							rested = {
								name = L["Rested"],
								desc = L["Set the colour of the rested bar."],
								type = "color",
								get = function()
									return self.db.profile.rested.r, self.db.profile.rested.g, self.db.profile.rested.b
								end,
								set = function(r, g, b)
									self.db.profile.rested.r, self.db.profile.rested.g, self.db.profile.rested.b = r, g, b
									self:UpdateXPBar()
								end,
								hasAlpha = false
							},
							resting = {
								name = L["Resting"],
								desc = L["Set the colour of the resting bar."],
								type = "color",
								get = function()
									return self.db.profile.resting.r, self.db.profile.resting.g, self.db.profile.resting.b
								end,
								set = function(r, g, b)
									self.db.profile.resting.r, self.db.profile.resting.g, self.db.profile.resting.b = r, g, b
									self:UpdateXPBar()
								end,
								hasAlpha = false
							},
							remaining = {
								name = L["Remaining"],
								desc = L["Set the colour of the remaining bar."],
								type = "color",
								get = function()
									return self.db.profile.remaining.r, self.db.profile.remaining.g, self.db.profile.remaining.b
								end,
								set = function(r, g, b)
									self.db.profile.remaining.r, self.db.profile.remaining.g, self.db.profile.remaining.b = r, g, b
									self:UpdateXPBar()
								end,
								hasAlpha = false
							},
						},
					},
				},
			},
			reputation = {
				name = L["Reputation"],
				desc = L["Reputation Bar related options"],
				type = "group",
				args = {
					showrep = {
						name = L["Show Reputation"],
						desc = L["Show the reputation bar instead of the XP bar."],
						type = "toggle",
						get = function() return self.db.profile.ShowReputation end,
						set = function(val) self.db.profile.ShowReputation = val self:UpdateXPBar() end,
					},
				},
			},
		},
	}
	XPBarNone:RegisterChatCommand({ "/xpbn", "/xpbarnone" }, self.options)

	self:CreateXPBar()
end

function XPBarNone:CreateXPBar()
	-- Make the XP Bar.
	XPBarNoneFrame = CreateFrame("Frame", "XPBarNoneFrame", UIParent)
	XPBarNoneFrame:SetFrameStrata(self.db.profile.Strata)
	XPBarNoneFrame:SetMovable(true)
	XPBarNoneFrame:Hide()

	if self.db.profile.Scale then XPBarNoneFrame:SetScale(self.db.profile.Scale) end

	XPBarNoneFrame:ClearAllPoints()
	if (not self.db.profile.PosX) then
		XPBarNoneFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	else
		XPBarNoneFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.PosX, self.db.profile.PosY)
	end
	XPBarNoneFrame:SetWidth(self.db.profile.Width)
	XPBarNoneFrame:SetHeight(self.db.profile.Height)
	
	XPBarNoneFrame:SetBackdrop({
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", edgeSize = 12,
		insets = {left = 5, right = 5, top = 5, bottom = 5}
	})
	self:ToggleBorder()

	local XPBarNoneButton = CreateFrame("Button", "XPBarNoneButton", XPBarNoneFrame)
	XPBarNoneButton:SetAllPoints(XPBarNoneFrame)
	XPBarNoneButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	XPBarNoneButton:RegisterForDrag("LeftButton")
	XPBarNoneButton:SetScript("OnClick", function()
		if (IsShiftKeyDown() and arg1 == "LeftButton") then
			if not ChatFrameEditBox:IsVisible() then ChatFrameEditBox:Show() end
			ChatFrameEditBox:Insert(XPBarNoneText:GetText())
		end
		if (IsShiftKeyDown() and arg1 == "RightButton") then
			dewdrop:Register(XPBarNoneButton, 'children', self.options, 'dontHook', true)
			dewdrop:Open(XPBarNoneButton, 'point', "TOP", 'relativePoint', "BOTTOM", 'cursorX', true)
			dewdrop:Unregister(XPBarNoneButton)
		end
		if (IsControlKeyDown() and arg1 == "RightButton") then
			dewdrop:Register(XPBarNoneButton, 'children', function() self:DewdropFactionsMenu() end, 'dontHook', true)
			dewdrop:Open(XPBarNoneButton, 'point', "TOP", 'relativePoint', "BOTTOM", 'cursorX', true)
			dewdrop:Unregister(XPBarNoneButton)
		end
	end)
	XPBarNoneButton:SetScript("OnDragStart", function ()
		if not self.db.profile.Locked then
			XPBarNoneFrame:StartMoving()
		end
	end)
	XPBarNoneButton:SetScript("OnDragStop", function()
		XPBarNoneFrame:StopMovingOrSizing()
		self:SavePosition()
	end)
        XPBarNoneButton:SetScript("OnEnter", function()
		if self.db.profile.MouseOver and not IsShiftKeyDown() and not IsControlKeyDown() then
			XPBarNone:ToggleShowReputation()
		else
			self.moshift = 1
		end
	end)
        XPBarNoneButton:SetScript("OnLeave", function()
		if self.db.profile.MouseOver and not self.moshift then
			XPBarNone:ToggleShowReputation()
		else
			self.moshift = nil
		end
	end)

	-- Frame Background
	local XPBarNoneB = CreateFrame("StatusBar", "XPBarNoneB", XPBarNoneFrame)
	XPBarNoneB:SetPoint("CENTER", XPBarNoneFrame, "CENTER", 0, 0)
	XPBarNoneB:SetWidth(XPBarNoneFrame:GetWidth() - 4)
	XPBarNoneB:SetHeight(XPBarNoneFrame:GetHeight() - 8)

	-- XP Bar
	local XPBarNoneA = CreateFrame("StatusBar", "XPBarNoneA", XPBarNoneFrame)
	XPBarNoneA:SetPoint("CENTER", XPBarNoneFrame, "CENTER", 0, 0)
	XPBarNoneA:SetWidth(XPBarNoneFrame:GetWidth() - 4)
	XPBarNoneA:SetHeight(XPBarNoneFrame:GetHeight() - 8)

	-- Remaining Rested XP Bar
	local XPBarNoneR = CreateFrame("StatusBar", "XPBarNoneR", XPBarNoneFrame)
	XPBarNoneR:SetPoint("CENTER", XPBarNoneFrame, "CENTER", 0, 0)
	XPBarNoneR:SetWidth(XPBarNoneFrame:GetWidth() - 4)
	XPBarNoneR:SetHeight(XPBarNoneFrame:GetHeight() - 8)

	-- Bubbles
	local XPBarNoneBubbles = CreateFrame("StatusBar", "XPBarNoneBubbles", XPBarNoneFrame)
	XPBarNoneBubbles:SetStatusBarTexture("Interface\\AddOns\\XPBarNone\\Textures\\bubbles")
	XPBarNoneBubbles:SetPoint("CENTER", XPBarNoneFrame, "CENTER", 0, 0)
	XPBarNoneBubbles:SetWidth(XPBarNoneFrame:GetWidth() - 4)
	XPBarNoneBubbles:SetHeight(XPBarNoneFrame:GetHeight() - 8)

	-- XP Bar Text
	local XPBarNoneText = XPBarNoneFrame:CreateFontString("XPBarNoneText", "ARTWORK")
	self:SetFontOptions()
	XPBarNoneText:SetShadowOffset(1, -1)
	XPBarNoneText:SetPoint("CENTER", XPBarNoneFrame, "CENTER")
	XPBarNoneText:SetTextColor(1,1,1,1)

	-- Set frame levels
	XPBarNoneB:SetFrameLevel(1)
	XPBarNoneR:SetFrameLevel(XPBarNoneB:GetFrameLevel() + 1)
	XPBarNoneA:SetFrameLevel(XPBarNoneR:GetFrameLevel() + 1)
	XPBarNoneBubbles:SetFrameLevel(XPBarNoneA:GetFrameLevel() + 1)
	XPBarNoneFrame:SetFrameLevel(XPBarNoneBubbles:GetFrameLevel() + 1)
	XPBarNoneButton:SetFrameLevel(XPBarNoneFrame:GetFrameLevel() + 1)

	-- Sort out the textures
	self:SetTexture(self.db.profile.Texture)

	-- Sort out the Bubbles
	self:ToggleBubbles()

	-- Restore the bar position
	self:RestorePosition()
end

function XPBarNone:OnEnable()
	self:RegisterEvent("PLAYER_XP_UPDATE", "UpdateXPData")
	self:RegisterEvent("PLAYER_LEVEL_UP", "UpdateXPData")
	self:RegisterEvent("UPDATE_EXHAUSTION", "UpdateXPBar")
	self:RegisterEvent("PLAYER_UPDATE_RESTING", "UpdateXPData")
	self:RegisterEvent("UPDATE_FACTION", "UpdateXPBar")
	self:RegisterEvent("Surface_Registered", "SurfacesUpdated")
	self:UpdateXPData()
	self:SurfacesUpdated()
	XPBarNoneFrame:Show()
end

function XPBarNone:OnDisable()
	XPBarNoneFrame:Hide()
end

function XPBarNone:ToggleBorder()
	if self.db.profile.Border then
		XPBarNoneFrame:SetBackdropBorderColor(1,1,1,1)
	else
		XPBarNoneFrame:SetBackdropBorderColor(0,0,0,0)
	end
end

function XPBarNone:ToggleBubbles()
	if self.db.profile.Bubbles then
		XPBarNoneBubbles:Show()
	else
		XPBarNoneBubbles:Hide()
	end
end

function XPBarNone:ToggleShowReputation()
	if self.db.profile.ShowReputation then
		self.db.profile.ShowReputation = false
		self:UpdateXPBar()
	else
		self.db.profile.ShowReputation = true
		self:UpdateXPBar()
	end
end

function XPBarNone:SetTexture(texture)
	XPBarNoneB:SetStatusBarTexture(surface:Fetch(texture))
	XPBarNoneA:SetStatusBarTexture(surface:Fetch(texture))
	XPBarNoneR:SetStatusBarTexture(surface:Fetch(texture))
	XPBarNoneB:SetStatusBarColor(0.5, 0.5, 0.5, 0.5)
end

function XPBarNone:SurfacesUpdated()
	self.options.args.general.args.texture.validate = surface:List()
end

function XPBarNone:SetFontOptions()
	local font, size, flags = GameFontNormal:GetFont()
	if self.db.profile.FontOutLine then
		flags = "OUTLINE"
	end
	XPBarNoneText:SetFont(font, self.db.profile.FontSize, flags)
end

function XPBarNone:UpdateXPData()
	local prevXP = self.cXP or 0

	self.cXP = UnitXP("player")
	self.nXP = UnitXPMax("player")
	self.remXP = self.nXP - self.cXP
	self.pcXP = self.cXP / self.nXP * 100
	self.diffXP = self.cXP - prevXP

	self:UpdateXPBar()
end

function XPBarNone:UpdateRepData()
	if not self.db.profile.ShowReputation then return end
	
	local repName, repStanding, repMin, repMax, repValue = GetWatchedFactionInfo()
	if (repName == nil) then
		if XPBarNoneA:IsVisible() then XPBarNoneA:Hide() end
		XPBarNoneText:SetText(L["You need to select a faction to watch."])
		return
	end

	repMax = repMax - repMin
	repValue = repValue - repMin
	repMin = 0
	local repPercent = repValue / repMax * 100

	local repText = string.format("Rep: %s (%s) %d/%d :: %.1f%%", repName, _G["FACTION_STANDING_LABEL"..repStanding], repValue, repMax, repPercent)
	
	if not XPBarNoneA:IsVisible() then XPBarNoneA:Show() end

	XPBarNoneR:Hide()
	XPBarNoneA:SetMinMaxValues(min(0, repValue), repMax)
	XPBarNoneA:SetValue(repValue)
	XPBarNoneA:SetStatusBarColor(FACTION_BAR_COLORS[repStanding].r, FACTION_BAR_COLORS[repStanding].g, FACTION_BAR_COLORS[repStanding].b)

	if not self.db.profile.HideText then
		XPBarNoneText:SetText(repText)
	else
		XPBarNoneText:SetText("")
	end
end

function XPBarNone:UpdateXPBar()
	if dewdrop:IsOpen(XPBarNoneButton) then
		dewdrop:Refresh(1)
	end
	if self.db.profile.ShowReputation then
		self:UpdateRepData()
		return
	else
		if not XPBarNoneA:IsVisible() then
			XPBarNoneA:Show()
		end
	end

	local restedXP = GetXPExhaustion()
	local xpText

	if (restedXP == nil) then
		if XPBarNoneR:IsVisible() then XPBarNoneR:Hide() end
		XPBarNoneA:SetStatusBarColor(self.db.profile.normal.r, self.db.profile.normal.g, self.db.profile.normal.b)
		xpText = string.format("Exp: %d/%d :: %.1f%% through level %d :: %d XP left", self.cXP, self.nXP, self.pcXP, UnitLevel("player"), self.remXP)
	else
		XPBarNoneR:SetMinMaxValues(min(0, self.cXP), self.nXP)
		XPBarNoneR:SetValue(self.cXP + restedXP)

		XPBarNoneR:SetStatusBarColor(self.db.profile.remaining.r, self.db.profile.remaining.g, self.db.profile.remaining.b)
		
		-- Do we want to indicate rest?
		if IsResting() and self.db.profile.IndicateRest then
			XPBarNoneA:SetStatusBarColor(self.db.profile.resting.r, self.db.profile.resting.g, self.db.profile.resting.b)
		else
			XPBarNoneA:SetStatusBarColor(self.db.profile.rested.r, self.db.profile.rested.g, self.db.profile.rested.b)
		end

		-- Are we showing the remaining xp?
		if self.db.profile.ShowRemaining then
				XPBarNoneR:Show()
		else
				XPBarNoneR:Hide()
		end

		-- Check if we want to show rested xp as a percentage
		if self.db.profile.RestedAsPercent then
			xpText = string.format("Exp: %d/%d (%.1f%%) :: %.1f%% through level %d :: %d XP left", self.cXP, self.nXP, restedXP / self.nXP * 100, self.pcXP, UnitLevel("player"), self.remXP)
		else
			xpText = string.format("Exp: %d/%d (%d) :: %.1f%% through level %d :: %d XP left", self.cXP, self.nXP, restedXP, self.pcXP, UnitLevel("player"), self.remXP)
		end
	end

	-- show how many kills to level?
	if self.db.profile.KillsToLevel then
		xpText = xpText .. string.format(" :: %d kills to level", math.ceil(self.remXP / self.diffXP))
	end

	XPBarNoneA:SetMinMaxValues(min(0, self.cXP), self.nXP)
	XPBarNoneA:SetValue(self.cXP)

	-- Are we hiding the text or not?
	if not self.db.profile.HideText then
		XPBarNoneText:SetText(xpText)
	else
		XPBarNoneText:SetText("")
	end
end

function XPBarNone:DewdropFactionsMenu()
	local name, standing, isHeader, isWatched, _ --, atWar
	-- Menu Header
	dewdrop:AddLine(
		'text', L["Faction Listing"],
		'isTitle', true,
		'justifyH', "CENTER",
		'notCheckable', true,
		'textHeight', 14,
		'textR', 1,
		'textG', 1,
		'textB', 1
	)

	-- Create menu
	for faction = 1, GetNumFactions() do
		name,_,standing,_,_,_,_,_,isHeader,isCollapsed,isWatched = GetFactionInfo(faction)

		if not isHeader then
			dewdrop:AddLine(
				'text', string.format("%s (%s)", name, _G["FACTION_STANDING_LABEL"..standing]),
				'textHeight', 12,
				'textR', FACTION_BAR_COLORS[standing].r,
				'textG', FACTION_BAR_COLORS[standing].g,
				'textB', FACTION_BAR_COLORS[standing].b,
				'checked', isWatched,
				'func', function(val)
					SetWatchedFactionIndex(val)
				end,
				'arg1', faction
			)
		else
			if isCollapsed then
				name = name.." (more)"
			end
			dewdrop:AddLine(
				'text', "- "..name.." -",
				'textHeight', 12,
				'notCheckable', true,
				'justifyH', "CENTER",
				'func', "ToggleCollapse",
				'arg1', self,
				'arg2', faction,
				'arg3', isCollapsed
			)
		end
	end

	-- Add a hint
	dewdrop:AddLine(
		'text', L["Hint: Click to set watched faction."],
		'notClickable', true,
		'notCheckable', true,
		'justifyH', "CENTER",
		'textR', 0,
		'textG', 1,
		'textB', 0
	)
end

function XPBarNone:ToggleCollapse(faction, isCollapsed)
	if isCollapsed then
		ExpandFactionHeader(faction)
	else
		CollapseFactionHeader(faction)
	end
end

function XPBarNone:SavePosition()
	local x,y = XPBarNoneFrame:GetLeft(), XPBarNoneFrame:GetTop()
	local s = XPBarNoneFrame:GetEffectiveScale()

	x,y = x*s, y*s

	self.db.profile.PosX = x
	self.db.profile.PosY = y
end

function XPBarNone:RestorePosition()
	local x = self.db.profile.PosX
	local y = self.db.profile.PosY
	if not x or not y then return end

	local s = XPBarNoneFrame:GetEffectiveScale()

	x,y = x/s, y/s

	XPBarNoneFrame:ClearAllPoints()
	XPBarNoneFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end
